var template = require('babel-template');

var mixinTpl = template(`
    mix(this, opts, MIXINS);
`);

var componentTpl = template(`
    Object.assign(CTOR.prototype, METHODS);

    ko.components.register(NAME, {
        synchronous: true,
        viewModel: {
            defaults: DEFAULTS,
            createViewModel: function (params, componentInfo) {
                return new CTOR(params, componentInfo);
            }
        },
        style: STYLE,
        template: TEMPLATE,
    });
`);

module.exports = function (babel) {
    var t = babel.types;

    var ast = null;

    // Captialize identifier
    var captialize = function (str) {
        if (str) {
            return str.charAt(0).toUpperCase() + str.substr(1);
        } else {
            return '';
        }
    };

    // Reset component partial ast
    var clean = function () {
        ast = {};
    };

    // Render component template into ast
    var render = function () {
        var constructorName = captialize(ast['name'].value);

        return componentTpl({
            CTOR: t.Identifier(constructorName),
            BODY: ast['body'],
            NAME: ast['name'],
            DEFAULTS: ast['defaults'],
            METHODS: ast['methods'],
            STYLE: ast['style'] || t.stringLiteral(''),
            TEMPLATE: ast['template'] || t.stringLiteral('')
        });
    };

    // Detect whether if a proper component module
    var isComponent = function () {
        return !t.isNullLiteral(ast['constructor']) &&
            t.isStringLiteral(ast['name']) &&
            ast['name'].value !== '';
    };

    var constructorVisitor = {
        FunctionExpression: {
            enter: function (path) {
                ast['body'] = path;
                path.node.id = t.identifier('Avatar');
            }
        },
        BlockStatement: {
            enter: function (path) {
                path.pushContainer('body', mixinTpl({
                    MIXINS: ast['mixins']
                }));
            }
        }
    };

    var variableVisitor = {
        VariableDeclarator: {
            enter: function (path) {
                var declarationName = path.node.id.name;

                switch (declarationName) {
                    case 'name':
                        ast[declarationName] = path.node.init;
                        break;
                    case 'constructor':
                        path.traverse(constructorVisitor);
                        break;
                    default:
                        ast[declarationName] = path.node.id;
                }
            }
        }
    };

    clean();

    return {
        visitor: {
            ExportNamedDeclaration: {
                enter: function (path) {
                    path.traverse(variableVisitor);
                }
            },
            Program: {
                exit: function (path) {
                    var declaration = null;

                    if (isComponent()) {
                        path.pushContainer('body', render());
                    }

                    clean();
                }
            }
        }
    };
};
